@@ -5,11 +5,12 @@ import tarfile
 import json
 from collections import namedtuple
 from itertools import chain
+from datetime import datetime
 
 from metaflow.metaflow_environment import MetaflowEnvironment
 from metaflow.exception import MetaflowNotFound,\
-                               MetaflowNamespaceMismatch,\
-                               MetaflowInternalError
+    MetaflowNamespaceMismatch,\
+    MetaflowInternalError
 
 from metaflow.metaflow_config import DEFAULT_METADATA
 from metaflow.plugins import ENVIRONMENTS, METADATA_PROVIDERS
@@ -65,7 +66,8 @@ def metadata(ms):
     infos = ms.split('@', 1)
     types = [m.TYPE for m in METADATA_PROVIDERS]
     if infos[0] in types:
-        current_metadata = [m for m in METADATA_PROVIDERS if m.TYPE == infos[0]][0]
+        current_metadata = [
+            m for m in METADATA_PROVIDERS if m.TYPE == infos[0]][0]
         if len(infos) > 1:
             current_metadata.INFO = infos[1]
     else:
@@ -247,6 +249,7 @@ class Metaflow(object):
         # run in this namespace. This is_in_namespace() function
         # does this properly in this case
         all_flows = self.metadata.get_object('root', 'flow')
+
         all_flows = all_flows if all_flows else []
         for flow in all_flows:
             try:
@@ -344,16 +347,20 @@ class MetaflowObject(object):
             raise MetaflowInternalError(msg="Unknown type: %s" % self._NAME)
 
         self._created_at = time.strftime(
-            '%Y-%m-%dT%H:%M:%SZ', time.gmtime(self._object['ts_epoch']//1000))
+            '%Y-%m-%dT%H:%M:%SZ', time.gmtime(self._object['ts_epoch']))
 
         self._tags = frozenset(chain(self._object.get('system_tags') or [],
                                      self._object.get('tags') or []))
 
+        self._aero_user = self._object.get("user_name", None)
+
         if _namespace_check and not self.is_in_namespace():
             raise MetaflowNamespaceMismatch(current_namespace)
 
     def _get_object(self, *path_components):
-        result = self._metaflow.metadata.get_object(self._NAME, 'self', None, *path_components)
+        result = self._metaflow.metadata.get_object(
+            self._NAME, 'self', None, *path_components)
+
         if not result:
             raise MetaflowNotFound("%s does not exist" % self)
         return result
@@ -369,9 +376,16 @@ class MetaflowObject(object):
         Iterator[MetaflowObject]
             Iterator over all children
         """
-        query_filter = {}
-        if current_namespace:
-            query_filter = {'any_tags': current_namespace}
+        if self._path_components is None:
+            flow_id = self._object.get('flow_id').lower()
+        else:
+            flow_id = self._path_components[0].lower()
+
+        if current_namespace == "production":
+            query_filter = {
+                'any_tags': f"production:{self._aero_user}_{flow_id}"}
+        else:
+            query_filter = {'any_tags': f"user:{self._aero_user}"}
 
         unfiltered_children = self._metaflow.metadata.get_object(
             self._NAME, _CLASSES[self._CHILD_CLASS]._NAME, query_filter, *self.path_components)
@@ -400,20 +414,20 @@ class MetaflowObject(object):
 
     def is_in_namespace(self):
         """
-        Returns whether this object is in the current namespace.
 
-        If the current namespace is None, this will always return True.
+        Aero: No longer requires check, as API filters based on user_name.
+        This function is used for filtering based on production tags.
 
         Returns
         -------
         bool
-            Whether or not the object is in the current namespace
+            Whether or not the object should be returned based on production stance
         """
+
         if self._NAME == 'flow':
             return any(True for _ in self)
-        else:
-            return current_namespace is None or\
-                   current_namespace in self._tags
+
+        return True
 
     def __str__(self):
         return "%s('%s')" % (self.__class__.__name__, self.pathspec)
@@ -519,7 +533,8 @@ class MetaflowObject(object):
             # We can skip the namespace check because if self._NAME = 'run',
             # the parent object is guaranteed to be in namespace.
             # Otherwise the check is moot for Flow since parent is singular.
-            self._parent = _CLASSES[self._PARENT_CLASS](parent_pathspec, _namespace_check=False)
+            self._parent = _CLASSES[self._PARENT_CLASS](
+                parent_pathspec, _namespace_check=False)
         return self._parent
 
     @property
@@ -1049,7 +1064,8 @@ class Task(MetaflowObject):
         env_type = my_code.info['environment_type']
         if not env_type:
             return None
-        env = [m for m in ENVIRONMENTS + [MetaflowEnvironment] if m.TYPE == env_type][0]
+        env = [m for m in ENVIRONMENTS +
+               [MetaflowEnvironment] if m.TYPE == env_type][0]
         return env.get_client_info(self.path_components[0], self.metadata_dict)
 
     def _load_log(self, stream):
