@@ -17,6 +17,7 @@ except NameError:
 from metaflow.exception import MetaflowException
 from metaflow.metaflow_config import AWS_SANDBOX_ENABLED, METADATA_SERVICE_AUTH_KEY
 
+
 class BatchClient(object):
     def __init__(self):
         from ..aws_client import get_aws_client
@@ -51,7 +52,7 @@ class BatchClient(object):
     def describe_job_queue(self, job_queue):
         paginator = self._client.get_paginator('describe_job_queues').paginate(
             jobQueues=[job_queue], maxResults=1)
-        return paginator.paginate()['jobQueues'][0] 
+        return paginator.paginate()['jobQueues'][0]
 
     def job(self):
         return BatchJob(self._client)
@@ -71,7 +72,7 @@ class BatchJobException(MetaflowException):
 class BatchJob(object):
     def __init__(self, client):
         self._client = client
-        tree = lambda: defaultdict(tree)
+        def tree(): return defaultdict(tree)
         self.payload = tree()
 
     def execute(self):
@@ -101,7 +102,7 @@ class BatchJob(object):
         self.payload['tags'] = {
             "UserId": aero_user
         }
-        
+
         response = self._client.submit_job(**self.payload)
         job = RunningJob(response['jobId'], self._client)
         return job.update()
@@ -126,13 +127,10 @@ class BatchJob(object):
             if len(response['jobQueues']) == 0:
                 raise BatchJobException(
                     'AWS Batch Job Queue %s not found.' % job_queue)
-            compute_environment = response['jobQueues'][0] \
-                                    ['computeEnvironmentOrder'][0] \
-                                    ['computeEnvironment']
+            compute_environment = response['jobQueues'][0]['computeEnvironmentOrder'][0]['computeEnvironment']
             response = self._client.describe_compute_environments(
                 computeEnvironments=[compute_environment])
-            platform = response['computeEnvironments'][0] \
-                            ['computeResources']['type']
+            platform = response['computeEnvironments'][0]['computeResources']['type']
 
         # compose job definition
         job_definition = {
@@ -166,12 +164,12 @@ class BatchJob(object):
             job_definition['platformCapabilities'] = ['FARGATE']
             job_definition['containerProperties']['networkConfiguration'] = \
                 {'assignPublicIp': 'ENABLED'}
-        
+
         # if platform == 'EC2' or platform == 'SPOT':
         #     if 'linuxParameters' not in job_definition['containerProperties']:
         #         job_definition['containerProperties']['linuxParameters'] = {}
         #     if shared_memory is not None:
-        #         if not (isinstance(shared_memory, (int, unicode, basestring)) and 
+        #         if not (isinstance(shared_memory, (int, unicode, basestring)) and
         #             int(shared_memory) > 0):
         #             raise BatchJobException(
         #                 'Invalid shared memory size value ({}); '
@@ -179,8 +177,8 @@ class BatchJob(object):
         #         else:
         #             job_definition['containerProperties'] \
         #                 ['linuxParameters']['sharedMemorySize'] = int(shared_memory)
-        #     if swappiness is not None: 
-        #         if not (isinstance(swappiness, (int, unicode, basestring)) and 
+        #     if swappiness is not None:
+        #         if not (isinstance(swappiness, (int, unicode, basestring)) and
         #             int(swappiness) >= 0 and int(swappiness) < 100):
         #             raise BatchJobException(
         #                 'Invalid swappiness value ({}); '
@@ -188,8 +186,8 @@ class BatchJob(object):
         #         else:
         #             job_definition['containerProperties'] \
         #                 ['linuxParameters']['swappiness'] = int(swappiness)
-        #     if max_swap is not None: 
-        #         if not (isinstance(max_swap, (int, unicode, basestring)) and 
+        #     if max_swap is not None:
+        #         if not (isinstance(max_swap, (int, unicode, basestring)) and
         #             int(max_swap) >= 0):
         #             raise BatchJobException(
         #                 'Invalid swappiness value ({}); '
@@ -288,7 +286,7 @@ class BatchJob(object):
         if 'resourceRequirements' not in self.payload['containerOverrides']:
             self.payload['containerOverrides']['resourceRequirements'] = []
         self.payload['containerOverrides']['resourceRequirements'].append(
-            {'value' : str(cpu), 'type': 'VCPU'}
+            {'value': str(cpu), 'type': 'VCPU'}
         )
         self.cpu_val = cpu
         return self
@@ -300,7 +298,7 @@ class BatchJob(object):
         if 'resourceRequirements' not in self.payload['containerOverrides']:
             self.payload['containerOverrides']['resourceRequirements'] = []
         self.payload['containerOverrides']['resourceRequirements'].append(
-            {'value' : str(mem), 'type': 'MEMORY'}
+            {'value': str(mem), 'type': 'MEMORY'}
         )
         self.memory_val = mem
         return self
@@ -345,6 +343,7 @@ class BatchJob(object):
         self.payload['retryStrategy']['attempts'] = attempts
         return self
 
+
 class Throttle(object):
     def __init__(self, delta_in_secs=1, num_tries=20):
         self.delta_in_secs = delta_in_secs
@@ -372,10 +371,12 @@ class Throttle(object):
                         random.randint(0, 3*self.delta_in_secs)
         return wrapped
 
+
 class TriableException(Exception):
     def __init__(self, ex):
         self.ex = ex
 
+
 class RunningJob(object):
 
     NUM_RETRIES = 8
@@ -493,8 +494,6 @@ class RunningJob(object):
         def get_log_stream(job):
             log_stream_name = job.log_stream_name
 
-            print(log_stream_name)
-
             if log_stream_name:
                 return BatchLogs('/aws/batch/job', log_stream_name, sleep_on_no_data=1)
             else:
@@ -509,7 +508,7 @@ class RunningJob(object):
                 self.wait_for_running()
 
         if log_stream is None:
-            return 
+            return
         exception = None
         for i in range(self.NUM_RETRIES + 1):
             try:
@@ -593,6 +592,7 @@ class BatchWaiter(object):
             jobs=[job_id]
         )
 
+
 class BatchLogs(object):
     def __init__(self, group, stream, pos=0, sleep_on_no_data=0):
         from ..aws_client import get_aws_client
@@ -643,4 +643,4 @@ class BatchLogs(object):
         events = self._get_events()
         for event in events:
             self._buf.append(event['message'])
-            self._pos = event['timestamp']
\ No newline at end of file
+            self._pos = event['timestamp']
