@@ -79,8 +79,7 @@ class StepFunctions(object):
             # that can translate cron specifications into a human readable 
             # format and push to the user for a better UX, someday.
             return 'This workflow triggers automatically '\
-                'via a cron schedule *%s* defined in Aero.' \
-                % self.name
+                'via a cron schedule defined in Aero.'
         else:
             return 'No triggers defined. '\
                 'You need to launch this workflow manually.'
