@@ -7,7 +7,8 @@ from distutils.version import LooseVersion
 from datetime import datetime
 
 from metaflow import current, decorators, parameters, JSONType
-from metaflow.metaflow_config import SFN_STATE_MACHINE_PREFIX
+from metaflow.metaflow_config import METADATA_SERVICE_AUTH_KEY
+
 from metaflow.exception import MetaflowException, MetaflowInternalError
 from metaflow.datastore.datastore import TransformableObject
 from metaflow.package import MetaflowPackage
@@ -28,7 +29,7 @@ class IncorrectMetadataServiceVersion(MetaflowException):
     headline = "Incorrect version for metaflow service"
 
 class StepFunctionsStateMachineNameTooLong(MetaflowException):
-    headline = "AWS Step Functions state machine name too long"
+    headline = "Aero Scheduler state machine name too long"
 
 @click.group()
 def cli():
@@ -92,7 +93,7 @@ def schedule(obj, name=None):
               help="Workflow timeout in seconds.")
 @click.option('--log-execution-history',
               is_flag=True,
-              help="Log AWS Step Functions execution history to AWS CloudWatch "
+              help="Log Aero Scheduler execution history to AWS CloudWatch "
                    "Logs log group.")
 @click.pass_obj
 def create(obj,
@@ -105,7 +106,7 @@ def create(obj,
            max_workers=None,
            workflow_timeout=None,
            log_execution_history=False):
-    obj.echo("Deploying *%s* to AWS Step Functions..." % obj.state_machine_name,
+    obj.echo("Deploying *%s* to Aero Scheduler..." % display_state_machine_name(obj.state_machine_name),
              bold=True)
 
     check_metadata_service_version(obj)
@@ -133,27 +134,32 @@ def create(obj,
         flow.deploy(log_execution_history)
         obj.echo("State Machine *{state_machine}* "
                  "for flow *{name}* pushed to "
-                 "AWS Step Functions successfully.\n"
-                    .format(state_machine=obj.state_machine_name,
+                 "Aero Scheduler successfully.\n"
+                    .format(state_machine=display_state_machine_name(obj.state_machine_name),
                             name=current.flow_name), 
                  bold=True)
         if obj._is_state_machine_name_hashed:
             obj.echo("Note that the flow was deployed with a truncated name "
-                     "due to a length limit on AWS Step Functions. The "
+                     "due to a length limit for Scheduled flows. The "
                      "original long name is stored in task metadata.\n")
         flow.schedule()
         obj.echo("What will trigger execution of the workflow:", bold=True)
         obj.echo(flow.trigger_explanation(), indent=True)
         
+def display_state_machine_name(name):
+    try:
+        split = name.split("_")
+        return "_".join(split[1:])
+    except:
+        return name
 
 def check_metadata_service_version(obj):
     return
 
 def resolve_state_machine_name(obj, name):
     def attach_prefix(name):
-      if SFN_STATE_MACHINE_PREFIX is not None:
-          return SFN_STATE_MACHINE_PREFIX + '_' + name
-      return name
+        aero_user = METADATA_SERVICE_AUTH_KEY.split(":")[0]
+        return aero_user + '_' + name
     project = current.get('project_name')
     obj._is_state_machine_name_hashed = False
     if project:
@@ -187,11 +193,11 @@ def resolve_state_machine_name(obj, name):
         is_project = False
 
         if len(state_machine_name) > 80:
-            msg = "The full name of the workflow:\n*%s*\nis longer than 80 "\
+            msg = "The full name of the workflow:\n*%s*\nis longer than 40 "\
                   "characters.\n\n"\
-                  "To deploy this workflow to AWS Step Functions, please "\
+                  "To deploy a Scheduled workflow, please "\
                   "assign a shorter name\nusing the option\n"\
-                  "*step-functions --name <name> create*." % state_machine_name
+                  "*schedule --name <name> create*." % state_machine_name
             raise StepFunctionsStateMachineNameTooLong(msg)
 
     return state_machine_name, token_prefix.lower(), is_project
@@ -252,7 +258,7 @@ def resolve_token(name,
     workflow = StepFunctions.get_existing_deployment(name)
     if workflow is None:
         obj.echo("It seems this is the first time you are deploying *%s* to "
-                 "the Aero scheduler." % name)
+                 "the Aero scheduler." % display_state_machine_name(name))
         prev_token = None
     else:
         prev_user, prev_token = workflow
@@ -267,19 +273,7 @@ def resolve_token(name,
         # we allow the user who deployed the previous version to re-deploy,
         # even if they don't have the token
         if prev_user != get_username() and authorize != prev_token:
-            obj.echo("There is an existing version of *%s* on the Aero "
-                     "scheduler which was deployed by the user "
-                     "*%s*." % (name, prev_user))
-            obj.echo("To deploy a new version of this flow, you need to use "
-                     "the same production token that they used. ")
-            obj.echo("Please reach out to them to get the token. Once you "
-                     "have it, call this command:")
-            obj.echo("    schedule create --authorize MY_TOKEN", 
-                        fg='green')
-            obj.echo('See "Organizing Results" at docs.metaflow.org for more '
-                     "information about production tokens.")
-            raise IncorrectProductionToken("Try again with the correct "
-                                           "production token.")
+            raise IncorrectProductionToken("A Flow with this name exists.")
 
     # 3) do we need a new token or should we use the existing token?
     if given_token:
@@ -306,23 +300,13 @@ def resolve_token(name,
                                         "Use --new-token to make a new "
                                         "namespace.")
         obj.echo('')
-        obj.echo("A new production token generated.")
     else:
         token = prev_token
 
     obj.echo('')
-    obj.echo("The namespace of this production flow is")
-    obj.echo('    production:%s' % token, fg='green')
-    obj.echo("To analyze results of this production flow "
-             "add this line in your notebooks:")
-    obj.echo('    namespace(\"production:%s\")' % token, fg='green')
-    obj.echo("If you want to authorize other people to deploy new versions "
-             "of this flow to the Aero scheduler, they need to call")
-    obj.echo("    step-functions create --authorize %s" % token, fg='green')
-    obj.echo("when deploying this flow to the Aero scheduler for the first "
-             "time.")
-    obj.echo('See "Organizing Results" at https://docs.metaflow.org/ for more '
-             "information about production tokens.")
+    obj.echo("To analyze results from a production flow, use")
+    obj.echo('    namespace("production")' , fg='green')
+    obj.echo("This will show results for any scheduled flows. ")
     obj.echo('')
     store_token(token_prefix, token)
     return token
@@ -345,9 +329,9 @@ def trigger(obj, **kwargs):
     response = StepFunctions.trigger(obj.state_machine_name, params)
 
     id = response['executionArn'].split(':')[-1]
-    obj.echo("Workflow *{name}* triggered on AWS Step Functions "
+    obj.echo("Workflow *{name}* triggered on Aero Scheduler "
         "(run-id *sfn-{id}*)."
-            .format(name=obj.state_machine_name, id=id), bold=True)
+            .format(name=display_state_machine_name(obj.state_machine_name), id=id), bold=True)
 
 
 @schedule.command(
@@ -422,11 +406,11 @@ def list_runs(obj,
                 else:
                     status += ', '
                 status += '*%s*' % state
-            obj.echo('No %s executions for *%s* found on AWS Step Functions.'
-                % (status, obj.state_machine_name))
+            obj.echo('No %s executions for *%s* found on Aero Scheduler.'
+                % (status, display_state_machine_name(obj.state_machine_name)))
         else:
-            obj.echo('No executions for *%s* found on AWS Step Functions.' \
-                % (obj.state_machine_name))
+            obj.echo('No executions for *%s* found on Aero Scheduler.' \
+                % display_state_machine_name(obj.state_machine_name))
 
 
 @schedule.command(
