@@ -169,7 +169,7 @@ class CondaStepDecorator(StepDecorator):
         return [files[0] for files in files]
 
     def _prepare_step_environment(self, step_name, ds_root):
-        env_id = self._resolve_step_environment(ds_root, True)
+        env_id = self._resolve_step_environment(ds_root)
         if env_id not in CondaStepDecorator.environments:
             cached_deps = read_conda_manifest(ds_root, self.flow.name)
             self.conda.create(self.step,
