@@ -1,9 +1,12 @@
 import os
 import requests
 import time
+import base64
+import json
 
 from distutils.version import LooseVersion
 
+from metaflow.plugins.aws.aws_client import get_signed_url, get_auth_object
 from metaflow.exception import MetaflowException
 from metaflow.metaflow_config import METADATA_SERVICE_NUM_RETRIES, METADATA_SERVICE_HEADERS, \
     METADATA_SERVICE_URL
@@ -13,10 +16,13 @@ from metaflow.sidecar import SidecarSubProcess
 from metaflow.sidecar_messages import MessageTypes, Message
 
 # Define message enums
+
+
 class HeartbeatTypes(object):
     RUN = 1
     TASK = 2
 
+
 class ServiceException(MetaflowException):
     headline = 'Metaflow service error'
 
@@ -30,18 +36,20 @@ class ServiceMetadataProvider(MetadataProvider):
     TYPE = 'service'
 
     def __init__(self, environment, flow, event_logger, monitor):
-        super(ServiceMetadataProvider, self).__init__(environment, flow, event_logger, monitor)
+        super(ServiceMetadataProvider, self).__init__(
+            environment, flow, event_logger, monitor)
         self.url_task_template = os.path.join(METADATA_SERVICE_URL,
-                                             'flows/{flow_id}/runs/{run_number}/steps/{step_name}/tasks/{task_id}/heartbeat')
+                                              'flows/{flow_id}/runs/{run_number}/steps/{step_name}/tasks/{task_id}/heartbeat')
         self.url_run_template = os.path.join(METADATA_SERVICE_URL,
-                                            'flows/{flow_id}/runs/{run_number}/heartbeat')
+                                             'flows/{flow_id}/runs/{run_number}/heartbeat')
         self.sidecar_process = None
 
     @classmethod
     def compute_info(cls, val):
         v = val.rstrip('/')
         try:
-            resp = requests.get(os.path.join(v, 'ping'), headers=METADATA_SERVICE_HEADERS)
+            resp = requests.get(os.path.join(v, 'ping'),
+                                headers=METADATA_SERVICE_HEADERS)
             resp.raise_for_status()
         except:  # noqa E722
             raise ValueError('Metaflow service [%s] unreachable.' % v)
@@ -107,9 +115,9 @@ class ServiceMetadataProvider(MetadataProvider):
         if heartbeat_type == HeartbeatTypes.TASK:
             # create task heartbeat
             data = {
-                    'flow_id': flow_id, 'run_number': run_id,
-                    'step_name': step_name, 'task_id': task_id,
-                    }
+                'flow_id': flow_id, 'run_number': run_id,
+                'step_name': step_name, 'task_id': task_id,
+            }
             payload[HB_URL_KEY] = self.url_task_template.format(**data)
         elif heartbeat_type == HeartbeatTypes.RUN:
             # create run heartbeat
@@ -145,13 +153,16 @@ class ServiceMetadataProvider(MetadataProvider):
                                 task_id,
                                 attempt_id,
                                 artifacts):
-        url = ServiceMetadataProvider._obj_path(self._flow_name, run_id, step_name, task_id)
+        url = ServiceMetadataProvider._obj_path(
+            self._flow_name, run_id, step_name, task_id)
         url += '/artifact'
-        data = self._artifacts_to_json(run_id, step_name, task_id, attempt_id, artifacts)
+        data = self._artifacts_to_json(
+            run_id, step_name, task_id, attempt_id, artifacts)
         self._request(self._monitor, url, data)
 
     def register_metadata(self, run_id, step_name, task_id, metadata):
-        url = ServiceMetadataProvider._obj_path(self._flow_name, run_id, step_name, task_id)
+        url = ServiceMetadataProvider._obj_path(
+            self._flow_name, run_id, step_name, task_id)
         url += '/metadata'
         data = self._metadata_to_json(run_id, step_name, task_id, metadata)
         self._request(self._monitor, url, data)
@@ -198,8 +209,10 @@ class ServiceMetadataProvider(MetadataProvider):
                   sys_tags=[]):
         # first ensure that the step exists
         self._get_or_create('step', run_id, step_name)
-        task = self._get_or_create('task', run_id, step_name, task_id, tags=tags, sys_tags=sys_tags)
-        self._register_code_package_metadata(run_id, step_name, task['task_id'])
+        task = self._get_or_create(
+            'task', run_id, step_name, task_id, tags=tags, sys_tags=sys_tags)
+        self._register_code_package_metadata(
+            run_id, step_name, task['task_id'])
         return task['task_id']
 
     @staticmethod
@@ -243,7 +256,8 @@ class ServiceMetadataProvider(MetadataProvider):
 
         always_create = False
         obj_path = self._obj_path(self._flow_name, run_id, step_name, task_id)
-        create_path = self._create_path(obj_type, self._flow_name, run_id, step_name)
+        create_path = self._create_path(
+            obj_type, self._flow_name, run_id, step_name)
         if obj_type == 'run' and run_id is None:
             always_create = True
         elif obj_type == 'task' and task_id is None:
@@ -260,26 +274,101 @@ class ServiceMetadataProvider(MetadataProvider):
             else:
                 raise
 
+    @classmethod
+    def _get_request_data(cls, http_function, url, headers, auth, data=None):
+        kwargs = {
+            "url": url,
+            "headers": headers,
+            "auth": auth
+        }
+
+        if data:
+            kwargs['json'] = data
+
+        data_responses = []
+        resp = http_function(**kwargs)
+        # Any error or other status code, return
+        if resp.status_code > 300:
+            return resp.text, resp.status_code
+
+        # Add data to the list of data responses
+        data_responses.append(resp.json()['data'])
+
+        # LEK = {task_id={N=2520}}
+
+        while 'LastEvaluatedKey' in resp.json():
+            # task_id=N=2520
+            base_last_evaluated_key = resp.json()['LastEvaluatedKey'].replace(
+                "}", "").replace("{", "")
+            split_last_evaluated_key = base_last_evaluated_key.split("=")
+
+            # Literally what is this, but it works. Python needs to add keys as variables for dict definition
+            exclusive_start_key = {}
+            exclusive_start_key[split_last_evaluated_key[0]] = {}
+            exclusive_start_key[split_last_evaluated_key[0]
+                                ][split_last_evaluated_key[1]] = split_last_evaluated_key[2]
+
+            kwargs['params'] = {"last_evaluated":
+                                base64.b64encode(
+                                    json.dumps(
+                                        exclusive_start_key
+                                    ).encode()
+                                )}
+
+            resp = http_function(**kwargs)
+            data_responses.append(resp.json()['data'])
+
+        if len(data_responses) == 1:
+            return data_responses[0], resp.status_code
+        else:
+            # Break out the list of lists into a single list
+            return [item for sublist in data_responses for item in sublist], resp.status_code
+
     @classmethod
     def _request(cls, monitor, path, data=None, retry_409_path=None):
         if cls.INFO is None:
             raise MetaflowException('Missing Metaflow Service URL. '
-                'Specify with METAFLOW_SERVICE_URL environment variable')
+                                    'Specify with METAFLOW_SERVICE_URL environment variable')
         url = os.path.join(cls.INFO, path.lstrip('/'))
+
+        auth = get_auth_object(url)
+        body = None
+        status_code = None
         for i in range(METADATA_SERVICE_NUM_RETRIES):
             try:
                 if data is None:
+
                     if monitor:
                         with monitor.measure('metaflow.service_metadata.get'):
-                            resp = requests.get(url, headers=METADATA_SERVICE_HEADERS)
+                            body, status_code = cls._get_request_data(
+                                requests.get, url, METADATA_SERVICE_HEADERS, auth)
+
                     else:
-                        resp = requests.get(url, headers=METADATA_SERVICE_HEADERS)
+                        body, status_code = cls._get_request_data(
+                            requests.get, url, METADATA_SERVICE_HEADERS, auth)
+
                 else:
+
                     if monitor:
                         with monitor.measure('metaflow.service_metadata.post'):
-                            resp = requests.post(url, headers=METADATA_SERVICE_HEADERS, json=data)
+                            resp = requests.post(
+                                url, headers=METADATA_SERVICE_HEADERS, json=data, auth=auth)
+
+                            status_code = resp.status_code
+                            if status_code < 300:
+                                body = resp.json()
+                            else:
+                                body = resp.text
+
                     else:
-                        resp = requests.post(url, headers=METADATA_SERVICE_HEADERS, json=data)
+                        resp = requests.post(
+                            url, headers=METADATA_SERVICE_HEADERS, json=data, auth=auth)
+
+                        status_code = resp.status_code
+                        if status_code < 300:
+                            body = resp.json()
+                        else:
+                            body = resp.text
             except:  # noqa E722
                 if monitor:
                     with monitor.count('metaflow.service_metadata.failed_request'):
@@ -288,11 +377,12 @@ class ServiceMetadataProvider(MetadataProvider):
                 else:
                     if i == METADATA_SERVICE_NUM_RETRIES - 1:
                         raise
-                resp = None
+
             else:
-                if resp.status_code < 300:
-                    return resp.json()
-                elif resp.status_code == 409 and data is not None:
+
+                if status_code < 300:
+                    return body
+                elif status_code == 409 and data is not None:
                     # a special case: the post fails due to a conflict
                     # this could occur when we missed a success response
                     # from the first POST request but the request
@@ -305,18 +395,18 @@ class ServiceMetadataProvider(MetadataProvider):
                         return cls._request(monitor, retry_409_path)
                     else:
                         return
-                elif resp.status_code != 503:
+                elif status_code != 503:
                     raise ServiceException('Metadata request (%s) failed (code %s): %s'
-                                           % (path, resp.status_code, resp.text),
-                                           resp.status_code,
-                                           resp.text)
+                                           % (path, status_code, body),
+                                           status_code,
+                                           body)
             time.sleep(2**i)
 
-        if resp:
+        if body:
             raise ServiceException('Metadata request (%s) failed (code %s): %s'
-                                   % (path, resp.status_code, resp.text),
-                                   resp.status_code,
-                                   resp.text)
+                                   % (path, status_code, body),
+                                   status_code,
+                                   body)
         else:
             raise ServiceException('Metadata request (%s) failed' % path)
 
