@@ -14,7 +14,7 @@ import fcntl
 import time
 import select
 import subprocess
-
+from datetime import datetime
 from io import BytesIO
 from functools import partial
 
@@ -28,7 +28,7 @@ from .datastore import DataException, MetaflowDatastoreSet
 from .metadata import MetaDatum
 from .debug import debug
 from .decorators import flow_decorators
-
+from .mflog import mflog, RUNTIME_LOG_SOURCE
 from .util import to_unicode, compress_list
 
 MAX_WORKERS=16
@@ -40,6 +40,10 @@ PROGRESS_INTERVAL = 1000 #ms
 # leveraging the MetaflowDatastoreSet.
 PREFETCH_DATA_ARTIFACTS = ['_foreach_stack', '_task_ok', '_transition']
 
+# Runtime must use logsource=RUNTIME_LOG_SOURCE for all loglines that it
+# formats according to mflog. See a comment in mflog.__init__
+mflog_msg = partial(mflog.decorate, RUNTIME_LOG_SOURCE)
+
 # TODO option: output dot graph periodically about execution
 
 class NativeRuntime(object):
@@ -60,8 +64,7 @@ class NativeRuntime(object):
                  clone_steps=None,
                  max_workers=MAX_WORKERS,
                  max_num_splits=MAX_NUM_SPLITS,
-                 max_log_size=MAX_LOG_SIZE,
-                 verbose_logging=False):
+                 max_log_size=MAX_LOG_SIZE):
 
         if run_id is None:
             self._run_id = metadata.new_run_id()
@@ -83,7 +86,6 @@ class NativeRuntime(object):
         self._entrypoint = entrypoint
         self.event_logger = event_logger
         self._monitor = monitor
-        self._verbose_logging = verbose_logging
 
         self._clone_run_id = clone_run_id
         self._clone_steps = {} if clone_steps is None else clone_steps
@@ -181,6 +183,8 @@ class NativeRuntime(object):
         self._logger('Flow starting (id %s):' % self._run_id,
                      system_msg=True)
 
+        self._metadata.start_run_heartbeat(self._flow.name, self._run_id)
+
         if self._params_task:
             self._queue_push('start', {'input_paths': [self._params_task]})
         else:
@@ -229,6 +233,8 @@ class NativeRuntime(object):
                 for deco in step.decorators:
                     deco.runtime_finished(exception)
 
+            self._metadata.stop_heartbeat()
+
         # assert that end was executed and it was successful
         if ('end', ()) in self._finished:
             self._logger('Done!', system_msg=True)
@@ -460,8 +466,7 @@ class Task(object):
                  join_type=None,
                  logger=None,
                  task_id=None,
-                 decos=[],
-                 verbose_logging=False):
+                 decos=[]):
         if task_id is None:
             task_id = str(metadata.new_task_id(run_id, step))
         else:
@@ -484,7 +489,6 @@ class Task(object):
         self.metadata = metadata
         self.event_logger = event_logger
         self.monitor = monitor
-        self.verbose_logging = verbose_logging
 
         self._logger = logger
         self._path = '%s/%s/%s' % (self.run_id, self.step, self.task_id)
@@ -538,19 +542,14 @@ class Task(object):
         self._ds.init_task()
 
 
-    def log(self, msg, system_msg=False, pid=None):
-        
-        if self.verbose_logging:
-            path = self._path
-        else:
-            path = "%s/%s" % (self.step, self.task_id)
+    def log(self, msg, system_msg=False, pid=None, timestamp=True):
 
-        if pid and self.verbose_logging:
-            prefix = '[%s (pid %s)] ' % (path, pid)
-        else:
-            prefix = '%s ' % path 
+        path = "%s/%s " % (self.step, self.task_id)
 
-        self._logger(msg, head=prefix, system_msg=system_msg)
+        self._logger(msg,
+                     head=path,
+                     system_msg=system_msg,
+                     timestamp=timestamp)
         sys.stdout.flush()
 
     def _find_origin_task(self, clone_run_id, join_type):
@@ -648,20 +647,11 @@ class Task(object):
         self._ds.persist(flow)
         self._ds.done()
 
-    def save_logs(self, logtype, logs):
-        location = self._ds.save_log(logtype, logs)
-        datum = [MetaDatum(field='log_location_%s' % logtype,
-                           value=json.dumps({
-                               'ds_type': self._ds.TYPE,
-                               'location': location,
-                               'attempt': self.retries}),
-                           type='log_path',
-                           tags=[])]
-        self.metadata.register_metadata(self.run_id,
-                                        self.step,
-                                        self.task_id,
-                                        datum)
-        return location
+    def save_logs(self, stdout_buffer, stderr_buffer):
+        self._ds.save_logs(RUNTIME_LOG_SOURCE, [
+            ('stdout', stdout_buffer),
+            ('stderr', stderr_buffer)
+        ])
 
     def save_metadata(self, name, metadata):
         self._ds.save_metadata(name, metadata)
@@ -701,7 +691,7 @@ class TruncatedBuffer(object):
                 self._size += len(bytedata)
             else:
                 msg = b'[TRUNCATED - MAXIMUM LOG FILE SIZE REACHED]\n'
-                self._buffer.write(msg)
+                self._buffer.write(mflog_msg(msg))
                 self._eof = True
 
     def get_bytes(self):
@@ -752,10 +742,18 @@ class CLIArgs(object):
 
     def get_args(self):
         def options(mapping):
+            """ Map a dictionary of options to a list of command-line arguments.
+
+            Values assigned to boolean values are assumed boolean flags. List values
+            are converted to repeated CLI arguments.
+
+            >>> list(options({"foo": True, "bar": False, "baz": "", "qux": ["a", "b"]}))
+            ["--foo", "--baz", "", "--qux", "a", "--qux", "b"]
+            """
             for k, v in mapping.items():
                 values = v if isinstance(v, list) else [v]
                 for value in values:
-                    if value:
+                    if value is not None and value is not False:
                         yield '--%s' % k
                         if not isinstance(value, bool):
                             yield to_unicode(value)
@@ -825,6 +823,7 @@ class Worker(object):
                                       self.task.retries,
                                       self.task.user_code_retries)
         env.update(args.get_env())
+        env['PYTHONUNBUFFERED'] = 'x'
         # the env vars are needed by the test framework, nothing else
         env['_METAFLOW_ATTEMPT'] = str(self.task.retries)
         if self.task.clone_run_id:
@@ -842,10 +841,38 @@ class Worker(object):
                                 stderr=subprocess.PIPE,
                                 stdout=subprocess.PIPE)
 
-    def write(self, msg, buf):
-        buf.write(msg)
-        text = msg.rstrip().decode(self._encoding, errors='replace')
-        self.task.log(text, pid=self._proc.pid)
+    def emit_log(self, msg, buf, system_msg=False):
+        if mflog.is_structured(msg):
+            res = mflog.parse(msg)
+            if res:
+                # parsing successful
+                plain = res.msg
+                timestamp = res.utc_tstamp
+                if res.should_persist:
+                    # in special circumstances we may receive structured
+                    # loglines that haven't been properly persisted upstream.
+                    # This is the case e.g. if a task crashes in an external
+                    # system and we retrieve the remaining logs after the crash.
+                    # Those lines are marked with a special tag, should_persist,
+                    # which we process here
+                    buf.write(mflog.unset_should_persist(msg))
+            else:
+                # parsing failed, corrupted logline. Print it as-is
+                timestamp = datetime.utcnow()
+                plain = msg
+        else:
+            # If the line isn't formatted with mflog already, we format it here.
+            plain = msg
+            timestamp = datetime.utcnow()
+            # store unformatted loglines in the buffer that will be
+            # persisted, assuming that all previously formatted loglines have
+            # been already persisted at the source.
+            buf.write(mflog_msg(msg, now=timestamp), system_msg=system_msg)
+        text = plain.strip().decode(self._encoding, errors='replace')
+        self.task.log(text,
+                      pid=self._proc.pid,
+                      timestamp=mflog.utc_to_local(timestamp),
+                      system_msg=system_msg)
 
     def read_logline(self, fd):
         fileobj, buf = self._logs[fd]
@@ -853,7 +880,7 @@ class Worker(object):
         # line buffering. If it does, things will deadlock
         line = fileobj.readline()
         if line:
-            self.write(line, buf)
+            self.emit_log(line, buf)
             return True
         else:
             return False
@@ -867,7 +894,8 @@ class Worker(object):
             return True
         if not self.cleaned:
             for fileobj, buf in self._logs.values():
-                buf.write(b'[KILLED BY ORCHESTRATOR]\n', system_msg=True)
+                msg = b'[KILLED BY ORCHESTRATOR]\n'
+                self.emit_log(msg, buf, system_msg=True)
             self.cleaned = True
         return self._proc.poll() is not None
 
@@ -905,17 +933,19 @@ class Worker(object):
         # Return early if the task is cloned since we don't want to
         # perform any log collection.
         if not self.task.is_cloned:
-            self.task.save_logs('stdout', self._stdout.get_bytes())
-            self.task.save_logs('stderr', self._stderr.get_bytes())
-
             self.task.save_metadata('runtime', {'return_code': returncode,
                                                 'killed': self.killed,
                                                 'success': returncode == 0})
             if returncode:
                 if not self.killed:
-                    self.task.log('Task failed.',
-                                  system_msg=True,
-                                  pid=self._proc.pid)
+                    if returncode == -11:
+                        self.emit_log(b'Task failed with a segmentation fault.',
+                                      self._stderr,
+                                      system_msg=True)
+                    else:
+                        self.emit_log(b'Task failed.',
+                                      self._stderr,
+                                      system_msg=True)
             else:
                 num = self.task.results['_foreach_num_splits']
                 if num:
@@ -925,7 +955,8 @@ class Worker(object):
                 self.task.log('Task finished successfully.',
                               system_msg=True,
                               pid=self._proc.pid)
-
+            self.task.save_logs(self._stdout.get_bytes(),
+                                self._stderr.get_bytes())
         return returncode
 
     def __str__(self):
